#' Guided-demo: Exploring 'ddmore' R package functionality of count data models
#' =========================================================================================
#' UseCase11 : Poisson distribution
#' ---------------------------------
#' This example script is intended to illustrate how to use the 'ddmore' R package
#' to perform a M&S workflow using the DDMoRe Standalone Execution Environment (SEE). 
#'   
#' The following steps are implemented in this workflow: 
#' * Initialisation of the R console
#' * Read and parse MDL files in R.
#' * Exploratory graphical analysis
#' * Parameter estimation with Monolix
#' * Evaluation of the model executed in Monolix using Xpose
#' * Parameter estimation of the same model with NONMEM
#' * Change estimation method to FOCEI and re-estimation of the model via PsN
#' * Bootstrap in Perl speaks NONMEM to evaluate parameter precision
#' * Updating parameter estimates in the MDL Parameter Object 
#'   using MLE values from NONMEM
#' * Performing a Visual Predictive Check (VPC) in PsN
#' * Using MLE values from NONMEM to simulate new observed values using the simulx function
#'   in the mlxR package.
#' 
#' To run a task, select with the cursor any code lines you wish to execute and press
#' CTRL+R+R in your keyboard.
#' An HTML file containing the commands in this file and associated output will be provided
#' to allow the user to compare the results


#' Initialisation
#' =========================
#+ Initialisation
#' Clear workspace and set working directory under 'UsesCasesDemo' project
rm(list=ls(all=FALSE))
mydir <- file.path(Sys.getenv("MDLIDE_WORKSPACE_HOME"),"UseCasesDemo")
setwd(mydir)

#' Create a working directory under 'models' folder where results are stored
uc<-"UseCase11"
datafile <- "count.csv"
mdlfile <- paste0(uc,".mdl")

#' Create a new folder to be used as working directory and where results will be stored
wd <- file.path(mydir,uc)
dir.create(wd)

#' Copy the dataset and the .mdl file available under "models" into the working directory
file.copy(file.path(mydir,"models", datafile),wd)
file.copy(file.path(mydir,"models",mdlfile),wd)

#' Set the working directory. 
setwd(file.path(mydir,uc))
#' The working directory needs to be set differently when knitr R package is used to spin the file 
library(knitr)
opts_knit$set(root.dir = file.path(mydir,uc)) 

#' List files available in working directory
list.files()



#' Introduction to 'ddmore' R package
#' ==================================
#' View objects within the .mdl file
#' ---------------------------
#' Use 'ddmore' function getMDLObjects() to retrieve model object(s) 
#' from an existing .mdl file. This function reads the MDL in an .mdl file
#' and parses the MDL code for each MDL Object into an R list of objects of
#' appropriate types with names corresponding to the MDL Object names given in the file.
#+ Read MDL file using the ddmore R package
myMDLObj <- getMDLObjects(mdlfile)
length(myMDLObj)
names(myMDLObj)

#' Use 'ddmore'  function getDataObjects() to retrieve only data object(s) from an existing .mdl file.
#' This function returns a list of Parameter Object(s) from which we select the first element.
#' Hover over the variable name to see its structure
myDataObj <- getDataObjects(mdlfile)[[1]]

#' Use 'ddmore'  function getParameterObjects() to retrieve only parameter object(s) from an existing .mdl file
myParObj <- getParameterObjects(mdlfile)[[1]]

#' Use 'ddmore'  function getModelObjects() to retrieve only model object(s) from an existing .mdl file.
myModObj <- getModelObjects(mdlfile)[[1]]

#' Use 'ddmore'  function getTaskPropertiesObjects() to retrieve only task properties object(s) from an existing .mdl file
myTaskObj <- getTaskPropertiesObjects(mdlfile)[[1]]



#' Exploratory Data Analysis
#' =========================
#' Recall that getDataObjects only reads the MDL code from the .mdl file.
#' Use 'ddmore' function readDataObj() to create an R object from the MDL data object.
#+ Exploratory Data Analysis
myData <- readDataObj(myDataObj)

#' Let's look at the first 6 lines of the data set
head(myData)


#' Extract only observation records
myEDAData<-myData[myData$MDV==0,]
head(myEDAData)

#' Open an R window to record and access all your plots
windows(record=TRUE) 

#' Now plot the data using xyplot from the lattice library 
plot1 <- xyplot(DV~TIME,groups=ID,data=myEDAData,type="b",ylab="Counts",xlab="Time (h)")
print(plot1)
plot2 <- xyplot(DV~CP,groups=ID,data=myEDAData,type="p",ylab="Counts",xlab="Conc")
print(plot2)
plot3 <- histogram(~DV, data=myEDAData, xlab="", type="count")
print(plot3)

#' Export results to a PDF file
pdf(paste0(uc,"_EGA.pdf"))
 print(plot1)
 print(plot2)
 print(plot3)
dev.off()

#' Compute mean and variances for each id
stats <- ddply(myEDAData, .(ID), summarise, mean=mean(DV), var=var(DV))

plot(xyplot(log(var)~log(mean),data=stats,
		main="Mean-Variance Plot",ylab="log(Individual variance)",xlab="log(Individual mean)"))

#' Compute the frequency of counts up to 10 for the Poissonness plot (Plan et al, CPT:PsP,2014)
library(plyr)
poiss <- count(myEDAData,"DV")
poiss$metameter<- log(poiss$freq) + lfactorial(poiss$DV) -log(sum(poiss$freq))

plot(xyplot(metameter~DV,data=poiss[poiss$DV<1000,],
		main="Poissonness plot",ylab="Count metameter",xlab="Count"))

#' How many observations per ID?
library(tables)
tabular(~(ID=as.factor(ID)), data=myEDAData)
tabular(~(DV=as.factor(DV)), data=myEDAData)



#' Model Development
#' =========================

#' ESTIMATE model parameters using Monolix
#' ------------------------- 
#' The ddmore "estimate" function translates the contents of the .mdl file to 
#' a target language and then estimates parameters using the target software. After estimation, 
#' the output is converted to a Standardised Output object which is saved in a .SO.xml file.
#'   
#' Translated files and Monolix output will be returned in the ./Monolix subfolder.
#' The Standardised Output object (.SO.xml) is read and parsed into an R object called "mlx" 
#' of (S4) class "StandardOutputObject".
#+ Monolix Estimation
mlx <- estimate(mdlfile, target="MONOLIX", subfolder="Monolix")

#' The ddmore "LoadSOObj" reads and parsed existing Standardise Output Objects
#' 
#'  mlx <- LoadSOObject("Monolix/UseCase11.SO.xml")
#' 
#' The ddmore function "getPopulationParameters" extracts the Population Parameter values from 
#' an R object of (S4) class "StandardOutputObject" and returns the estimates. 
#' See documentation for getPopulationParameters to see other arguments and settings for this function.  
parameters_mlx <- getPopulationParameters(mlx, what="estimates")$MLE
print(parameters_mlx)
print(getPopulationParameters(mlx, what="precisions"))

print(getEstimationInfo(mlx))

#' SAEM Estimation with NONMEM
#' -------------------------
#' By default, a covariance step is not run when estimating in NONMEM. 
#' To see how it can be requested, see UseCase1_1.mdl
#+ NONMEM Estimation SAEM
NM <- estimate(mdlfile, target="NONMEM", subfolder="NONMEM")

#' Load previous results
#' 
#'  NM <- LoadSOObject("NONMEM/UseCase11.SO.xml")
#' 
#' The ddmore function "getPopulationParameters" extracts the Population Parameter values from 
#' an R object of (S4) class "StandardOutputObject" and returns the estimates. 
#' See documentation for getPopulationParameters to see other arguments and settings for this function.  
#' Results from NONMEM should be comparable with results from MONOLIX
parameters_nm <- getPopulationParameters(NM, what="estimates")
print(parameters_nm)
print(parameters_mlx)

print(getEstimationInfo(NM))


#' Xpose diagnostics using NONMEM output
#+ GOF for NONMEM Estimation
#' -------------------------
#' There is a bug in the generation of xpose database
#nm.xpdb<-as.xpdb(NM,datafile)
#
##' Perform some basic goodness of fit 
#print(parm.hist(nm.xpdb, inclZeroWRES=TRUE))
#
##' Export graphs to a PDF file
#pdf("GOF_NM.pdf")
# print(parm.hist(nm.xpdb, inclZeroWRES=TRUE))
#dev.off()



#' Change estimation method to FOCEI (for speed)
#' ------------------------------------------------------------
#+ Change_estimation_method
#' MDL Objects can be manipulated from R to change for example the estimation algorithm
myTaskProperties <- getTaskPropertiesObjects(mdlfile)[[1]]
myNewTaskProperties <- myTaskProperties
myNewTaskProperties@ESTIMATE$algo <- "foce"

#' Assembling the new Modelling Object Group (MOG). 
#' Note that we reuse the data, parameters and model from the MOG. 
myNewerMOG <- createMogObj(dataObj = getDataObjects(mdlfile)[[1]], 
		parObj = getParameterObjects(mdlfile)[[1]], 
		mdlObj = getModelObjects(mdlfile)[[1]], 
		taskObj = myNewTaskProperties)

#' We can then write the MOG back out to an .mdl file.
mdlfile.FOCE <- paste0(uc,"_FOCE.mdl")
writeMogObj(myNewerMOG,mdlfile.FOCE)

#' Test estimation using this new MOG in NONMEM via PsN
#' 
#' By default, a covariance step is not run when estimating in PsN. 
#' To see how it can be requested, see UseCase1_1.mdl
#+ PsN (NONMEM) Estimation FOCEI
NM.FOCE <- estimate(mdlfile.FOCE, target="PsN", subfolder="NONMEM_FOCE")

#' The ddmore "LoadSOObj" reads and parsed existing Standardise Output Objects
#' 
#' NM.FOCE <- LoadSOObject("NONMEM_FOCE/UseCase11_FOCE.SO.xml")
#' 
#' Results from NONMEM should be comparable to previous results
getPopulationParameters(NM.FOCE,  what="estimates")$MLE
parameters_nm
parameters_mlx


#' Run the bootstrap using PsN
#' ------------------------------------
#' The ddmore "bootstrap.PsN" function is a wrap up function that calls Bootstrap PsN
#' functionality using as input an MDL file that will be translated to NMTRAN as first step.
#' Additional PsN arguments can be specified under the "bootstrapOptions" attribute.
#' After task execution, the output from PsN is converted to a Standardised Output object
#'  which is saved in a .SO.xml file.
#' Translated files and PsN output will be returned in the ./Bootstrap subfolder
#+ Bootstrap
bootstrapResults <- bootstrap.PsN(mdlfile.FOCE, samples=20, seed=12345,
		bootstrapOptions=" -no-skip_minimization_terminated -threads=2",
		subfolder="Bootstrap", plot=TRUE)

#' Load results from a bootstrap previously performed
#' 
#' bootstrapResults <-  LoadSOObject("Bootstrap/UseCase11.SO.xml")
#' 
#' Export bootstrap histograms to a pdf
pdf(paste0(uc,"_Bootstrap.pdf"))
print(boot.hist(results.file = file.path("Bootstrap",paste0("raw_results_",uc,"_FOCE.csv")),
				incl.ids.file = file.path("Bootstrap","included_individuals1.csv")))
dev.off()

#' Extract parameter estimates and precision from bootstrap results.
getPopulationParameters(bootstrapResults, what="estimates")

#' Extract the information regarding the precision intervals
getPopulationParameters(bootstrapResults,what="intervals")$Bootstrap



#' VPC of model
#' --------------------------------
#' Before running the VPC with PsN we must update the (initial) values in the MDL Parameter Object.
#' MLE estimates from previous step can be used 
#+ Update parObj
structuralPar <- getPopulationParameters(NM.FOCE, what="estimates",block='structural')$MLE
variabilityPar <- getPopulationParameters(NM.FOCE, what="estimates",block='variability')$MLE



#' Update the parameter object using the ddmore "updateParObj" function.
#' This function updates an R object of (S4) class "parObj".
#' The user chooses which block to update, what items within that block,
#' and what to replace those items with. 
#' NOTE: that updateParObj can only update attributes which ALREADY EXIST in the MDL Parameter Object for that item.
#' This ensures that valid MDL is preserved.
myParObj <- getParameterObjects(mdlfile)[[1]]
myParObjUpdated <- updateParObj(myParObj,block="STRUCTURAL",
		item=names(structuralPar),
		with=list(value=structuralPar))
myParObjUpdated <- updateParObj(myParObjUpdated,block="VARIABILITY",
		item=names(variabilityPar),
		with=list(value=variabilityPar))


#' Check that the appropriate initial values have been updated to the MLE values from the previous fit.
myParObjUpdated@STRUCTURAL
myParObjUpdated@VARIABILITY


#' Assembling the new MOG. Note that we reuse the data and model from the previous run.  
myVPCMOG <- createMogObj(dataObj = getDataObjects(mdlfile)[[1]], 
		parObj = myParObjUpdated, 
		mdlObj = getModelObjects(mdlfile)[[1]], 
		taskObj = getTaskPropertiesObjects(mdlfile)[[1]])

#' We can then write the MOG back out to an .mdl file.
mdlfile.VPC <- paste0(uc,"_VPC.mdl")
writeMogObj(myVPCMOG,mdlfile.VPC)


#' Similarly as above, ddmore "VPC.PsN" function can be used to run a VPC using PsN as target tool.
#' CP is used as idv to perform a VPC
vpcFiles_CP <- VPC.PsN(mdlfile.VPC,samples=20, seed=12345,
		vpcOptions ="-n_simulation=10 -flip_comments -idv=CP",
		subfolder="VPC_CP", plot=TRUE) 

#' To replay the visualisation using information from the VPC SO file
pdf(paste0(uc,"_VPC.pdf"))
  xpose.VPC(vpc.info=file.path("./VPC_CP", vpcFiles_CP@RawResults@DataFiles$PsN_VPC_results$path),
 		vpctab=file.path("./VPC_CP", vpcFiles_CP@RawResults@DataFiles$PsN_VPC_vpctab$path),
 		main="",
		logy=TRUE)
dev.off()



#' Simulation using simulx
#' --------------------------------
#' The mlxR package has been developed to visualize and explore models that are encoded in
#' MLXTRAN or PharmML 
#' The ddmore function as.PharmML translates an MDL file (extension .mdl) to its PharmML
#' representation. The output file (extension .xml) is saved in the working directory.
myPharmML <- as.PharmML(mdlfile)

#' Use parameter values from the FOCEI estimation
p <- getPopulationParameters(NM.FOCE, what="estimates")$MLE

#' Parameter values used in simulation
print(p)

#' Simulate PK parameters for individuals
ind <- list(name = c('BASECOUNT','BETA'))

#' Simulate Poisson intensity (LAMBDA) and observed counts (Y)
f   <- list( name = c('LAMBDA'), time = 0:2)
y   <- list( name = c('Y'), time = 0:2)

#' Simulate 20 subjects
g <- list( size = 20, level = 'longitudinal')

#' Call simulx
res  <- simulx(model = myPharmML,
		       parameter = p,
			   group = g,
			   output = list(ind,f,y))

#' Simulated parameter values for each individual
print(res$parameter)

#' Plot simulated results
print(ggplotmlx(aes(x=time, y=LAMBDA), data=res$LAMBDA) + geom_line(size=1) +
				geom_point(aes(x=time, y=Y, color = id), data=res$Y, color="red") + ylab("") )

