#' Guided-demo: Exploring 'ddmore' R package functionality of time to event models
#' =========================================================================================
#' UseCase14_2 : ODE + time to event model for exact and right censored information
#' ---------------------------------
#' This example script is intended to illustrate how to use the 'ddmore' R package
#' to perform a M&S workflow using the DDMoRe Standalone Execution Environment (SEE). 
#'   
#' The following steps are implemented in this workflow: 
#' * Initialisation of the R console
#' * Parameter estimation with NONMEM
#' * Change estimation method to FOCEI and re-estimation of the model via PsN
#' * Bootstrap in Perl speaks NONMEM to evaluate parameter precision
#' 
#' To run a task, select with the cursor any code lines you wish to execute and press
#' CTRL+R+R in your keyboard.
#' An HTML file containing the commands in this file and associated output will be provided
#' to allow the user to compare the results


#' Initialisation
#' =========================
#+ Initialisation
#' Clear workspace and set working directory under 'UsesCasesDemo' project
rm(list=ls(all=FALSE))
mydir <- file.path(Sys.getenv("MDLIDE_WORKSPACE_HOME"),"UseCasesDemo")
setwd(mydir)

#' Create a working directory under 'models' folder where results are stored
uc<-"UseCase14_2"
datafile <- "warfarin_TTE_exact.csv"
mdlfile <- paste0(uc,".mdl")

#' Create a new folder to be used as working directory and where results will be stored
wd <- file.path(mydir,uc)
dir.create(wd)

#' Copy the dataset and the .mdl file available under "models" into the working directory
file.copy(file.path(mydir,"models", datafile),wd)
file.copy(file.path(mydir,"models",mdlfile),wd)

#' Set the working directory. 
setwd(file.path(mydir,uc))
#' The working directory needs to be set differently when knitr R package is used to spin the file 
library(knitr)
opts_knit$set(root.dir = file.path(mydir,uc)) 

#' List files available in working directory
list.files()


#' Model Development
#' =========================

#' ESTIMATE model parameters using Monolix
#' ------------------------- 
#' Monolix estimation is not possible for this use case. In the current version of the framework, 
#' there is no place to define initial conditions for an ODE which are not structural parameters so that the model runs interoperably.
#' See UC14_1 for an alternative implementation that can be used with Monolix. 


#' SAEM Estimation with NONMEM
#' -------------------------
#' By default, a covariance step is not run when estimating in NONMEM. 
#' To see how it can be requested, see UseCase1_1.mdl
#+ NONMEM Estimation SAEM
NM <- estimate(mdlfile, target="NONMEM", subfolder="NONMEM")

#' Load previous results
#' 
#'  NM <- LoadSOObject("NONMEM/UseCase14_2.SO.xml")
#' 
#' The ddmore function "getPopulationParameters" extracts the Population Parameter values from 
#' an R object of (S4) class "StandardOutputObject" and returns the estimates. 
#' See documentation for getPopulationParameters to see other arguments and settings for this function.  
#' Results from NONMEM should be comparable with results from MONOLIX
parameters_nm <- getPopulationParameters(NM, what="estimates")
print(parameters_nm)

print(getEstimationInfo(NM))

#' Change estimation method to FOCEI (for speed)
#' ------------------------------------------------------------
#+ Change_estimation_method
#' MDL Objects can be manipulated from R to change for example the estimation algorithm
myTaskProperties <- getTaskPropertiesObjects(mdlfile)[[1]]
myNewTaskProperties <- myTaskProperties
myNewTaskProperties@ESTIMATE$algo <- "foce"

#' Assembling the new Modelling Object Group (MOG). 
#' Note that we reuse the data, parameters and model from the MOG. 
myNewerMOG <- createMogObj(dataObj = getDataObjects(mdlfile)[[1]], 
		parObj = getParameterObjects(mdlfile)[[1]], 
		mdlObj = getModelObjects(mdlfile)[[1]], 
		taskObj = myNewTaskProperties)

#' We can then write the MOG back out to an .mdl file.
mdlfile.FOCE <- paste0(uc,"_FOCE.mdl")
writeMogObj(myNewerMOG,mdlfile.FOCE)

#' Test estimation using this new MOG in NONMEM via PsN
#' 
#' By default, a covariance step is not run when estimating in PsN. 
#' To see how it can be requested, see UseCase1_1.mdl
#+ PsN (NONMEM) Estimation FOCEI
NM.FOCE <- estimate(mdlfile.FOCE, target="PsN", subfolder="NONMEM_FOCE")

#' The ddmore "LoadSOObj" reads and parsed existing Standardise Output Objects
#' 
#' NM.FOCE <- LoadSOObject("NONMEM_FOCE/UseCase14_2_FOCE.SO.xml")
#' 
#' Results from NONMEM should be comparable to previous results
print(getPopulationParameters(NM.FOCE,  what="estimates")$MLE)
print(parameters_nm)

