#' UseCase1_1 : ODE model for warfarin population pharmacokinetics (extended task properties)
#' -------------------------------------------------------------
#' This example script is intended to illustrate how to use the 'ddmore' R package
#' to perform a M&S workflow using the DDMoRe Standalone Execution Environment (SEE). 
#'   
#' The following steps are implemented in this workflow: 
#' * Initialisation of the R console
#' * Read and parse MDL files in R.
#' * Parameter estimation with Monolix
#' * Parameter estimation of the same model with NONMEM
#' * Change estimation method to FOCEI and re-estimation of the model via PsN
#'   
#' To run a task, select wiht the coursor any code lines you wish to execute and press
#' CTRL+R+R in your keyboard.
#' An HTML file containing the commands in this file and associated output will be provided
#' to allow the user to compare the results


#' Initialisation
#' =========================
#' Clear workspace and set working directory under 'UsesCasesDemo' project
#+ Initialisation
rm(list=ls(all=F))
mydir <- file.path(Sys.getenv("MDLIDE_WORKSPACE_HOME"),"UseCasesDemo")
setwd(mydir)

#' Set name of .mdl file and dataset for future tasks
uc <- "UseCase1_1"
datafile <- "warfarin_conc.csv"
mdlfile <- paste0(uc,".mdl")

#' Create a new folder to be used as working directory and where results will be stored
wd <- file.path(mydir,uc)
dir.create(wd)

#' Copy the dataset and the .mdl file available under "models" into the working directory
file.copy(file.path(mydir,"models", datafile),wd)
file.copy(file.path(mydir,"models",mdlfile),wd)

#' Set the working directory. 
setwd(file.path(mydir,uc))
#' The working directory needs to be set differently when knitr R package is used to spin the file 
library(knitr)
opts_knit$set(root.dir = file.path(mydir,uc))
cache.path <- file.path(mydir,uc, 'cache')
dir.create(cache.path)
opts_chunk$set(cache.path=cache.path)
figure.path <- file.path(mydir,uc, 'figure')
dir.create(figure.path)
opts_chunk$set(figure.path=figure.path)

#' List files available in working directory
list.files()


#' Introduction to 'ddmore' R package
#' ==================================
#' View objects within the .mdl file
#' ---------------------------
#' Use 'ddmore' function getMDLObjects() to retrieve model object(s) 
#' from an existing .mdl file. This function reads the MDL in an .mdl file
#' and parses the MDL code for each MDL Object into an R list of objects of
#' appropriate types with names corresponding to the MDL Object names given in the file.
#+ Read MDL file using the ddmore R package
myMDLObj <- getMDLObjects(mdlfile)
length(myMDLObj)
names(myMDLObj)

#' Use 'ddmore'  function getDataObjects() to retrieve only data object(s) from an existing .mdl file.
#' This function returns a list of Parameter Object(s) from which we select the first element
#' Hover over the variable name to see its structure
myDataObj <- getDataObjects(mdlfile)[[1]]

#' Use 'ddmore'  function getParameterObjects() to retrieve only parameter object(s) from an existing .mdl file
myParObj <- getParameterObjects(mdlfile)[[1]]

#' Use 'ddmore'  function getModelObjects() to retrieve only model object(s) from an existing .mdl file.
myModObj <- getModelObjects(mdlfile)[[1]]

#' Use 'ddmore'  function getTaskPropertiesObjects() to retrieve only task properties object(s) from an existing .mdl file
myTaskObj <- getTaskPropertiesObjects(mdlfile)[[1]]
#' Objects can also be retrieved by name
myTaskObj.FOCEI <- getTaskPropertiesObjects(mdlfile)[["FOCEI_task"]]
myTaskObj.SAEM <- getTaskPropertiesObjects(mdlfile)[["SAEM_task"]]

#' Model Development
#' =========================

#' ESTIMATE model parameters using Monolix
#' -------------------------
#' The ddmore "estimate" function translates the contents of the .mdl file to 
#' a target language and then estimates parameters using the target software. After estimation, 
#' the output is converted to a Standardised Output object which is saved in a .SO.xml file.
#'   
#' Translated files and Monolix output will be returned in the ./Monolix subfolder.
#' The Standardised Output object (.SO.xml) is read and parsed into an R object called "mlx" 
#' of (S4) class "StandardOutputObject".
#+ Monolix Estimation, cache=TRUE
mlx <- estimate(mdlfile, target="MONOLIX", subfolder="Monolix")

#' Checking that the Monolix task settings have been provided to the MLXTRAN Project file correctly. 
myTaskObj.SAEM

scan(file.path(getwd(),"Monolix",uc,paste(uc,"_project.mlxtran",sep="")), strip.white=T, what="character", sep="\n")

#' SAEM Estimation with NONMEM
#' -------------------------
#+ NONMEM Estimation SAEM, cache=TRUE
NM <- estimate(mdlfile, target="NONMEM", subfolder="NONMEM")

myTaskObj.SAEM

scan(file.path(getwd(),"NONMEM",paste(uc,".ctl",sep="")), what="character", sep="\n", strip.white=T)

#' Change estimation method to FOCEI (for speed)
#' -----------------------------------------------
#' Assembling the new Modelling Object Group (MOG). 
#' Note that we reuse the data, parameters and model from the MOG and only task properties is changed  
#+ Change Estimation Method
myNewerMOG <- createMogObj(dataObj = getDataObjects(mdlfile)[[1]], 
		parObj = getParameterObjects(mdlfile)[[1]], 
		mdlObj = getModelObjects(mdlfile)[[1]], 
		taskObj = getTaskPropertiesObjects(mdlfile)[["FOCEI_task"]],
		info=list(problemStmt="Warfarin Pop PK base model", name="UseCase1 variant 1"))

#' We can then write the MOG back out to an .mdl file.
mdlfile.FOCEI <- paste0(uc,"_FOCEI.mdl")
writeMogObj(myNewerMOG,mdlfile.FOCEI)

#' Test estimation using this new MOG in NONMEM via PsN
#+ PsN (NONMEM) Estimation FOCEI, cache=TRUE
NM.FOCEI <- estimate(mdlfile.FOCEI, target="PsN", subfolder="NONMEM_FOCEI")
#' Load previous results
#' NM.FOCEI <- LoadSOObject("NONMEM_FOCEI/UseCase1_1_FOCEI.SO.xml")

myTaskObj.FOCEI

scan(file.path(getwd(),"NONMEM_FOCEI",paste(uc,"_FOCEI.ctl",sep="")), what="character", sep="\n", strip.white=T)

#' Covariance step
getPopulationParameters(NM, what="precisions") 
